"use strict";

exports.__esModule = true;
exports.getActiveModality = getActiveModality;
exports.getModality = getModality;
exports.addModalityListener = addModalityListener;
exports.testOnly_resetActiveModality = testOnly_resetActiveModality;

var _createEventHandle = _interopRequireDefault(require("../create-event-handle"));

var _supportsDOM = _interopRequireDefault(require("./supportsDOM"));

var _supportsPointerEvent = _interopRequireDefault(require("./supportsPointerEvent"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright (c) Nicolas Gallagher.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var activeModality = 'keyboard';
var modality = 'keyboard';
var previousModality;
var previousActiveModality;
var isEmulatingMouseEvents = false;
var listeners = new Set();
var KEYBOARD = 'keyboard';
var MOUSE = 'mouse';
var TOUCH = 'touch';
var BLUR = 'blur';
var CONTEXTMENU = 'contextmenu';
var FOCUS = 'focus';
var KEYDOWN = 'keydown';
var MOUSEDOWN = 'mousedown';
var MOUSEMOVE = 'mousemove';
var MOUSEUP = 'mouseup';
var POINTERDOWN = 'pointerdown';
var POINTERMOVE = 'pointermove';
var SCROLL = 'scroll';
var SELECTIONCHANGE = 'selectionchange';
var TOUCHCANCEL = 'touchcancel';
var TOUCHMOVE = 'touchmove';
var TOUCHSTART = 'touchstart';
var VISIBILITYCHANGE = 'visibilitychange';
var bubbleOptions = {
  passive: true
};
var captureOptions = {
  capture: true,
  passive: true
}; // Window events

var blurHandle = (0, _createEventHandle.default)(BLUR, bubbleOptions);
var focusHandle = (0, _createEventHandle.default)(FOCUS, bubbleOptions); // Must be capture phase because 'stopPropagation' might prevent these
// events bubbling to the document.

var visibilityChangeHandle = (0, _createEventHandle.default)(VISIBILITYCHANGE, captureOptions);
var keyDownHandle = (0, _createEventHandle.default)(KEYDOWN, captureOptions);
var pointerDownHandle = (0, _createEventHandle.default)(POINTERDOWN, captureOptions);
var pointerMoveHandle = (0, _createEventHandle.default)(POINTERMOVE, captureOptions); // Fallback events

var contextMenuHandle = (0, _createEventHandle.default)(CONTEXTMENU, captureOptions);
var mouseDownHandle = (0, _createEventHandle.default)(MOUSEDOWN, captureOptions);
var mouseMoveHandle = (0, _createEventHandle.default)(MOUSEMOVE, captureOptions);
var mouseUpHandle = (0, _createEventHandle.default)(MOUSEUP, captureOptions);
var scrollHandle = (0, _createEventHandle.default)(SCROLL, captureOptions);
var selectiomChangeHandle = (0, _createEventHandle.default)(SELECTIONCHANGE, captureOptions);
var touchCancelHandle = (0, _createEventHandle.default)(TOUCHCANCEL, captureOptions);
var touchMoveHandle = (0, _createEventHandle.default)(TOUCHMOVE, captureOptions);
var touchStartHandle = (0, _createEventHandle.default)(TOUCHSTART, captureOptions);

function restoreModality() {
  if (previousModality != null || previousActiveModality != null) {
    if (previousModality != null) {
      modality = previousModality;
      previousModality = null;
    }

    if (previousActiveModality != null) {
      activeModality = previousActiveModality;
      previousActiveModality = null;
    }

    callListeners();
  }
}

function onBlurWindow() {
  previousModality = modality;
  previousActiveModality = activeModality;
  activeModality = KEYBOARD;
  modality = KEYBOARD;
  callListeners(); // for fallback events

  isEmulatingMouseEvents = false;
}

function onFocusWindow() {
  restoreModality();
}

function onKeyDown(event) {
  if (event.metaKey || event.altKey || event.ctrlKey) {
    return;
  }

  if (modality !== KEYBOARD) {
    modality = KEYBOARD;
    activeModality = KEYBOARD;
    callListeners();
  }
}

function onVisibilityChange() {
  if (document.visibilityState !== 'hidden') {
    restoreModality();
  }
}

function onPointerish(event) {
  var eventType = event.type;

  if ((0, _supportsPointerEvent.default)()) {
    if (eventType === POINTERDOWN) {
      if (activeModality !== event.pointerType) {
        modality = event.pointerType;
        activeModality = event.pointerType;
        callListeners();
      }

      return;
    }

    if (eventType === POINTERMOVE) {
      if (modality !== event.pointerType) {
        modality = event.pointerType;
        callListeners();
      }

      return;
    }
  } // Fallback for non-PointerEvent environment
  else {
      if (!isEmulatingMouseEvents) {
        if (eventType === MOUSEDOWN) {
          if (activeModality !== MOUSE) {
            modality = MOUSE;
            activeModality = MOUSE;
            callListeners();
          }
        }

        if (eventType === MOUSEMOVE) {
          if (modality !== MOUSE) {
            modality = MOUSE;
            callListeners();
          }
        }
      } // Flag when browser may produce emulated events


      if (eventType === TOUCHSTART) {
        isEmulatingMouseEvents = true;

        if (event.touches && event.touches.length > 1) {
          isEmulatingMouseEvents = false;
        }

        if (activeModality !== TOUCH) {
          modality = TOUCH;
          activeModality = TOUCH;
          callListeners();
        }

        return;
      } // Remove flag after emulated events are finished or cancelled, and if an
      // event occurs that cuts short a touch event sequence.


      if (eventType === CONTEXTMENU || eventType === MOUSEUP || eventType === SELECTIONCHANGE || eventType === SCROLL || eventType === TOUCHCANCEL || eventType === TOUCHMOVE) {
        isEmulatingMouseEvents = false;
      }
    }
}

if ((0, _supportsDOM.default)()) {
  blurHandle.setListener(window, onBlurWindow);
  focusHandle.setListener(window, onFocusWindow);
  keyDownHandle.setListener(document, onKeyDown);
  pointerDownHandle.setListener(document, onPointerish);
  pointerMoveHandle.setListener(document, onPointerish);
  visibilityChangeHandle.setListener(document, onVisibilityChange); // fallbacks

  contextMenuHandle.setListener(document, onPointerish);
  mouseDownHandle.setListener(document, onPointerish);
  mouseMoveHandle.setListener(document, onPointerish);
  mouseUpHandle.setListener(document, onPointerish);
  touchCancelHandle.setListener(document, onPointerish);
  touchMoveHandle.setListener(document, onPointerish);
  touchStartHandle.setListener(document, onPointerish);
  selectiomChangeHandle.setListener(document, onPointerish);
  scrollHandle.setListener(document, onPointerish);
}

function callListeners() {
  var value = {
    activeModality: activeModality,
    modality: modality
  };
  listeners.forEach(function (listener) {
    listener(value);
  });
}

function getActiveModality() {
  return activeModality;
}

function getModality() {
  return modality;
}

function addModalityListener(listener) {
  listeners.add(listener);
  return function () {
    listeners.delete(listener);
  };
}

function testOnly_resetActiveModality() {
  isEmulatingMouseEvents = false;
  activeModality = KEYBOARD;
  modality = KEYBOARD;
}